@@ -77,7 +77,7 @@
  */
 static command_rec ssl_config_cmds[] = {
     /*
-     * Per-server context configuration directives
+     * Global (main-server) context configuration directives
      */
     AP_SRV_CMD(Mutex, TAKE1, 
                "SSL lock for handling internal mutual exclusions "
@@ -85,6 +85,13 @@ static command_rec ssl_config_cmds[] = {
     AP_SRV_CMD(PassPhraseDialog, TAKE1, 
                "SSL dialog mechanism for the pass phrase query "
                "(`builtin', `exec:/path/to/program')")
+    AP_SRV_CMD(SessionCache, TAKE1,
+               "SSL Session Cache storage "
+               "(`none', `dbm:/path/to/file')")
+
+    /*
+     * Per-server context configuration directives
+     */
     AP_SRV_CMD(Engine, FLAG,
                "SSL switch for the protocol engine "
                "(`on', `off')")
@@ -103,18 +110,12 @@ static command_rec ssl_config_cmds[] = {
     AP_SRV_CMD(CACertificateFile, TAKE1,
                "SSL CA Certificate file "
                "(`/path/to/file' - PEM encoded)")
-    AP_SRV_CMD(CACertificateReqFile, TAKE1,
-               "SSL CA Certificate SSLv3-Request file "
-               "(`/path/to/file' - PEM encoded)")
     AP_SRV_CMD(VerifyClient, TAKE1,
                "SSL Client verify type "
                "(`none', `optional', `require', `optional_no_ca')")
     AP_SRV_CMD(VerifyDepth, TAKE1,
                "SSL Client verify depth "
                "(`N' - number of intermediate certifcates)")
-    AP_SRV_CMD(SessionCache, TAKE1,
-               "SSL Session Cache storage mode "
-               "(`none', `dbm:/path/to/file')")
     AP_SRV_CMD(SessionCacheTimeout, TAKE1,
                "SSL Session Cache object lifetime "
                "(`N' - number of seconds)")
