@@ -194,17 +194,6 @@
 #define SSL_SESSION_CACHE_TIMEOUT  300
 #endif
 
-/* 
- * Set to TRUE if you want to allow SSL to be disabled on a per-request basis
- * (useful for subrequests) 
- */
-#define SSL_IS_OPTIONAL TRUE  
-#if SSL_IS_OPTIONAL
-#define OPTIONAL_SSL    (ap_ctx_get(r->connection->client->ctx, "ssl") == NULL)
-#else
-#define OPTIONAL_SSL    FALSE
-#endif
-
 /* 
  * Support for file locking: Try to determine whether we should use fcntl() or
  * flock().  Would be better ap_config.h could provide this... :-(
