@@ -72,114 +72,155 @@
  * The mapping of obsolete directives to official ones...
  */
 
-static char *ssl_compat_SSLRequireCipher(pool *p, const char *oline);
-static char *ssl_compat_SSLBanCipher(pool *p, const char *oline);
-static char *ssl_compat_words2list(pool *p, const char *oline);
+static char *ssl_compat_SSLRequireCipher(pool *p, const char *, const char *, const char *);
+static char *ssl_compat_SSLBanCipher(pool *p, const char *, const char *, const char *);
+static char *ssl_compat_words2list(pool *p, const char *);
+
+#define CRM_BEGIN              /* nop */
+#define CRM_ENTRY(what,action) { what, action },
+#define CRM_END                { NULL, NULL, NULL, NULL, NULL, NULL }
+#define CRM_CMD(cmd)           cmd, NULL, NULL
+#define CRM_STR(str)           NULL, str, NULL
+#define CRM_PAT(cmd)           NULL, NULL, pat
+#define CRM_LOG(msg)           msg, NULL, NULL 
+#define CRM_SUB(new)           NULL, new, NULL
+#define CRM_CAL(fct)           NULL, NULL, fct
 
 static struct {
-    enum { CRM_END, CRM_TXTSUB, CRM_TXTMSG, CRM_PATFCT } nType;
-    char *cpMatch;
-    void *vpSubst;
+    char *cpCommand;
+    char *cpSubstring;
+    char *cpPattern;
+    char *cpMessage;
+    char *cpSubst;
+    char *(*fpSubst)(pool *, const char *, const char *, const char *);
 } ssl_cmd_rewrite_map[] = {
+    CRM_BEGIN
+
     /* 
      * Apache-SSL 1.x & mod_ssl 2.0.x backward compatibility
      */
-    { CRM_TXTSUB, "SSLEnable",                   "SSLEngine on"                },
-    { CRM_TXTSUB, "SSLDisable",                  "SSLEngine off"               },
-    { CRM_TXTSUB, "SSLLogFile",                  "SSLLog"                      },
-    { CRM_TXTSUB, "SSLRequiredCiphers",          "SSLCipherSuite"              },
-    { CRM_PATFCT, "SSLRequireCipher",            ssl_compat_SSLRequireCipher   },
-    { CRM_PATFCT, "SSLBanCipher",                ssl_compat_SSLBanCipher       },
-    { CRM_TXTSUB, "SSLFakeBasicAuth",            "SSLOptions +FakeBasicAuth"   },
-    { CRM_TXTSUB, "SSLExportClientCertificates", "SSLOptions +ExportCertData"  },
-    { CRM_TXTMSG, "SSLCacheServerPath",          "Use SSLSessionCache instead" },
-    { CRM_TXTMSG, "SSLCacheServerPort",          "Use SSLSessionCache instead" },
-    { CRM_TXTMSG, "SSLCacheServerRunDir",        "Not needed for mod_ssl"      },
+    CRM_ENTRY( CRM_CMD("SSLEnable"),                   CRM_SUB("SSLEngine on")                )
+    CRM_ENTRY( CRM_CMD("SSLDisable"),                  CRM_SUB("SSLEngine off")               )
+    CRM_ENTRY( CRM_CMD("SSLLogFile"),                  CRM_SUB("SSLLog")                      )
+    CRM_ENTRY( CRM_CMD("SSLRequiredCiphers"),          CRM_SUB("SSLCipherSuite")              )
+    CRM_ENTRY( CRM_CMD("SSLRequireCipher"),            CRM_CAL(ssl_compat_SSLRequireCipher)   )
+    CRM_ENTRY( CRM_CMD("SSLBanCipher"),                CRM_CAL(ssl_compat_SSLBanCipher)       )
+    CRM_ENTRY( CRM_CMD("SSLFakeBasicAuth"),            CRM_SUB("SSLOptions +FakeBasicAuth")   )
+    CRM_ENTRY( CRM_CMD("SSLCacheServerPath"),          CRM_LOG("Use SSLSessionCache instead") )
+    CRM_ENTRY( CRM_CMD("SSLCacheServerPort"),          CRM_LOG("Use SSLSessionCache instead") )
+
+    /* 
+     * Apache-SSL 1.x backward compatibility
+     */
+    CRM_ENTRY( CRM_CMD("SSLExportClientCertificates"), CRM_SUB("SSLOptions +ExportCertData")  )
+    CRM_ENTRY( CRM_CMD("SSLCacheServerRunDir"),        CRM_LOG("Not needed for mod_ssl")      )
+
     /* 
      * Sioux 1.x backward compatibility
      */
-    { CRM_TXTSUB, "SSL_CertFile",           "SSLCertificateFile"               },
-    { CRM_TXTSUB, "SSL_KeyFile",            "SSLCertificateKeyFile"            },
-    { CRM_TXTSUB, "SSL_CipherSuite",        "SSLCipherSuite"                   },
-    { CRM_TXTSUB, "SSL_X509VerifyDir",      "SSLCACertificatePath"             },
-    { CRM_TXTSUB, "SSL_Log",                "SSLLogFile"                       },
-    { CRM_TXTSUB, "SSL_Connect"             "SSLEngine"                        },
-    { CRM_TXTSUB, "SSL_ClientAuth"          "SSLVerifyClient"                  },
-    { CRM_TXTSUB, "SSL_X509VerifyDepth"     "SSLVerifyDepth"                   },
-    { CRM_TXTMSG, "SSL_FetchKeyPhraseFrom", "Use SSLPassPhraseDialog instead"  },
-    { CRM_TXTMSG, "SSL_SessionDir",         "Use SSLSessionCache instead"      },
-    { CRM_TXTMSG, "SSL_Require",            "Use SSLRequire instead (Syntax!)" },
-    { CRM_TXTMSG, "SSL_CertFileType",       "Not supported by mod_ssl"         },
-    { CRM_TXTMSG, "SSL_KeyFileType",        "Not supported by mod_ssl"         },
-    { CRM_TXTMSG, "SSL_X509VerifyPolicy",   "Not supported by mod_ssl"         },
-    { CRM_TXTMSG, "SSL_LogX509Attributes",  "Not supported by mod_ssl"         },
+    CRM_ENTRY( CRM_CMD("SSL_CertFile"),            CRM_SUB("SSLCertificateFile")              )
+    CRM_ENTRY( CRM_CMD("SSL_KeyFile"),             CRM_SUB("SSLCertificateKeyFile")           )
+    CRM_ENTRY( CRM_CMD("SSL_CipherSuite"),         CRM_SUB("SSLCipherSuite")                  )
+    CRM_ENTRY( CRM_CMD("SSL_X509VerifyDir"),       CRM_SUB("SSLCACertificatePath")            )
+    CRM_ENTRY( CRM_CMD("SSL_Log"),                 CRM_SUB("SSLLogFile")                      )
+    CRM_ENTRY( CRM_CMD("SSL_Connect"),             CRM_SUB("SSLEngine")                       )
+    CRM_ENTRY( CRM_CMD("SSL_ClientAuth"),          CRM_SUB("SSLVerifyClient")                 )
+    CRM_ENTRY( CRM_CMD("SSL_X509VerifyDepth"),     CRM_SUB("SSLVerifyDepth")                  )
+    CRM_ENTRY( CRM_CMD("SSL_FetchKeyPhraseFrom"),  CRM_LOG("Use SSLPassPhraseDialog instead") )
+    CRM_ENTRY( CRM_CMD("SSL_SessionDir"),          CRM_LOG("Use SSLSessionCache instead")     )
+    CRM_ENTRY( CRM_CMD("SSL_Require"),             CRM_LOG("Use SSLRequire instead (Syntax!)"))
+    CRM_ENTRY( CRM_CMD("SSL_CertFileType"),        CRM_LOG("Not supported by mod_ssl")        )
+    CRM_ENTRY( CRM_CMD("SSL_KeyFileType"),         CRM_LOG("Not supported by mod_ssl")        )
+    CRM_ENTRY( CRM_CMD("SSL_X509VerifyPolicy"),    CRM_LOG("Not supported by mod_ssl")        )
+    CRM_ENTRY( CRM_CMD("SSL_LogX509Attributes"),   CRM_LOG("Not supported by mod_ssl")        )
+
     /* 
      * Stronghold 2.x backward compatibility
      */
-    { CRM_TXTSUB, "SSLFlag",                   "SSLEngine"                        },
-    { CRM_TXTSUB, "SSLSessionLockFile",        "SSLMutex"                         },
-    { CRM_TXTSUB, "SSLLogFile",                "SSLLog"                           },
-    { CRM_TXTSUB, "RequireSSL",                "SSLRequireSSL"                    },
-    { CRM_TXTSUB, "SSLCipherList",             "SSLCipherSuite"                   },
-    { CRM_TXTMSG, "SSLErrorFile",              "Not needed for mod_ssl"           },
-    { CRM_TXTMSG, "SSLProtocol",               "Not supported by mod_ssl"         },
-    { CRM_TXTMSG, "SSLRoot",                   "Not supported by mod_ssl"         },
-    { CRM_TXTMSG, "SSL_CertificateLogDir",     "Not supported by mod_ssl"         },
-    { CRM_TXTMSG, "AuthCertDir",               "Not supported by mod_ssl"         },
-    { CRM_TXTMSG, "SSL_Group",                 "Not supported by mod_ssl"         },
-    { CRM_TXTMSG, "SSLProxyMachineCertPath",   "Not supported by mod_ssl"         },
-    { CRM_TXTMSG, "SSLProxyMachineCertFile",   "Not supported by mod_ssl"         },
-    { CRM_TXTMSG, "SSLProxyCACertificatePath", "Not supported by mod_ssl"         },
-    { CRM_TXTMSG, "SSLProxyCACertificateFile", "Not supported by mod_ssl"         },
-    { CRM_TXTMSG, "SSLProxyVerifyDepth",       "Not supported by mod_ssl"         },
-    { CRM_TXTMSG, "SSLProxyCipherList",        "Not supported by mod_ssl"         },
-    /*
-     * End of map.
-     */
-    { CRM_END, NULL, NULL }
+    CRM_ENTRY( CRM_CMD("SSLFlag"),                   CRM_SUB("SSLEngine")                     )
+    CRM_ENTRY( CRM_CMD("SSLSessionLockFile"),        CRM_SUB("SSLMutex")                      )
+    CRM_ENTRY( CRM_CMD("RequireSSL"),                CRM_SUB("SSLRequireSSL")                 )
+    CRM_ENTRY( CRM_CMD("SSLCipherList"),             CRM_SUB("SSLCipherSuite")                )
+    CRM_ENTRY( CRM_CMD("SSLErrorFile"),              CRM_LOG("Not needed for mod_ssl")        )
+    CRM_ENTRY( CRM_CMD("SSLProtocol"),               CRM_LOG("Not supported by mod_ssl")      )
+    CRM_ENTRY( CRM_CMD("SSLRoot"),                   CRM_LOG("Not supported by mod_ssl")      )
+    CRM_ENTRY( CRM_CMD("SSL_CertificateLogDir"),     CRM_LOG("Not supported by mod_ssl")      )
+    CRM_ENTRY( CRM_CMD("AuthCertDir"),               CRM_LOG("Not supported by mod_ssl")      )
+    CRM_ENTRY( CRM_CMD("SSL_Group"),                 CRM_LOG("Not supported by mod_ssl")      )
+    CRM_ENTRY( CRM_CMD("SSLProxyMachineCertPath"),   CRM_LOG("Not supported by mod_ssl")      )
+    CRM_ENTRY( CRM_CMD("SSLProxyMachineCertFile"),   CRM_LOG("Not supported by mod_ssl")      )
+    CRM_ENTRY( CRM_CMD("SSLProxyCACertificatePath"), CRM_LOG("Not supported by mod_ssl")      )
+    CRM_ENTRY( CRM_CMD("SSLProxyCACertificateFile"), CRM_LOG("Not supported by mod_ssl")      )
+    CRM_ENTRY( CRM_CMD("SSLProxyVerifyDepth"),       CRM_LOG("Not supported by mod_ssl")      )
+    CRM_ENTRY( CRM_CMD("SSLProxyCipherList"),        CRM_LOG("Not supported by mod_ssl")      )
+
+    CRM_END
 };
 
-static char *ssl_compat_SSLRequireCipher(pool *p, const char *oline)
+static char *ssl_compat_SSLRequireCipher(pool *p, const char *oline, const char *cmd, const char *args)
 {
-    return ap_pstrcat(p, "SSLRequire %{SSL_CIPHER} in (", 
-                          ssl_compat_words2list(p, oline),
-                          ")", NULL); 
+    return ap_pstrcat(p, "SSLRequire %{SSL_CIPHER} in {", 
+                          ssl_compat_words2list(p, args),
+                          "}", NULL); 
 }
 
-static char *ssl_compat_SSLBanCipher(pool *p, const char *oline)
+static char *ssl_compat_SSLBanCipher(pool *p, const char *oline, const char *cmd, const char *args)
 {
-    return ap_pstrcat(p, "SSLRequire not (%{SSL_CIPHER} in (", 
-                          ssl_compat_words2list(p, oline),
-                          "))", NULL); 
+    return ap_pstrcat(p, "SSLRequire not (%{SSL_CIPHER} in {", 
+                          ssl_compat_words2list(p, args),
+                          "})", NULL); 
 }
 
 static char *ssl_compat_words2list(pool *p, const char *oline)
 {
     char *line;
+    char *cpB;
+    char *cpE;
     char *cpI;
     char *cpO;
     char n;
 
-    for (cpI = (char *)oline, n = 1; *cpI != NUL; )
+    /*
+     * Step 1: Determine borders
+     */
+    cpB = (char *)oline;
+    while (*cpB == ' ' || *cpB == '\t')
+       cpB++;
+    cpE = cpB+strlen(cpB);
+    while (cpE > cpB && (*(cpE-1) == ' ' || *(cpE-1) == '\t'))
+        cpE--;
+
+    /*
+     * Step 2: Determine final size and allocate buffer
+     */
+    for (cpI = cpB, n = 1; cpI < cpE; cpI++)
         if ((*cpI == ' ' || *cpI == '\t') && 
-            (cpI > oline && *(cpI-1) != ' ' && *(cpI-1) != '\t'))
+            (cpI > cpB && *(cpI-1) != ' ' && *(cpI-1) != '\t'))
             n++;
-    line = ap_palloc(p, strlen(oline)+(n*2)+n+1);
-    cpI = (char *)oline;
+    line = ap_palloc(p, (cpE-cpB)+(n*2)+n+1);
+    cpI = cpB;
     cpO = line;
-    *cpO++ = '"';
-    while (*cpI != NUL) {
-        if (*cpI == ' ' || *cpI == '\t') {
+    while (cpI < cpE) {
+        if (   (*cpI != ' ' && *cpI != '\t') 
+            && (   cpI == cpB
+                || (   cpI > cpB 
+                    && (*(cpI-1) == ' ' || *(cpI-1) == '\t')))) {
+            *cpO++ = '"';
+            *cpO++ = *cpI++;
+        }
+        else if (   (*cpI == ' ' || *cpI == '\t') 
+                 && (   cpI > cpB 
+                     && (*(cpI-1) != ' ' && *(cpI-1) != '\t'))) {
             *cpO++ = '"';
             *cpO++ = ',';
+            *cpO++ = *cpI++;
         }
-        if ((*cpI != ' ' && *cpI != '\t') && 
-             cpI > oline && (*(cpI-1) == ' ' || *(cpI-1) == '\t')) {
-            *cpO++ = '"';
+        else {
+            *cpO++ = *cpI++;
         }
-        *cpO++ = *cpI++;
     }
-    *cpO++ = '"';
+    if (cpI > cpB && (*(cpI-1) != ' ' && *(cpI-1) != '\t'))
+        *cpO++ = '"';
     *cpO++ = NUL;
     return line;
 }
@@ -189,6 +230,9 @@ char *ssl_compat_directive(server_rec *s, pool *p, const char *oline)
     int i;
     char *line;
     char *cp;
+    char caCmd[1024];
+    char *cpArgs;
+    int match;
 
     /*
      * Skip comment lines
@@ -200,37 +244,69 @@ char *ssl_compat_directive(server_rec *s, pool *p, const char *oline)
         return NULL;
 
     /*
-     * Apply rewriting map to directives
+     * Extract directive name
+     */
+    cp = (char *)oline;
+    for (i = 0; *cp != ' ' && *cp != '\t' && *cp != NUL && i < 1024; )
+        caCmd[i++] = *cp++;
+    caCmd[i] = NUL;
+    cpArgs = cp;
+
+    /*
+     * Apply rewriting map
      */
     line = NULL;
-    for (i = 0; ssl_cmd_rewrite_map[i].nType != CRM_END; i++) {
-        if (ssl_cmd_rewrite_map[i].nType == CRM_TXTSUB) {
-            if ((cp = strstr(oline, ssl_cmd_rewrite_map[i].cpMatch)) != NULL) {
-                line = ssl_util_ptxtsub(p, oline, ssl_cmd_rewrite_map[i].cpMatch,
-                                        (char *)(ssl_cmd_rewrite_map[i].vpSubst));
-                break;
-            }
+    for (i = 0; !(ssl_cmd_rewrite_map[i].cpCommand == NULL &&
+                  ssl_cmd_rewrite_map[i].cpPattern == NULL   ); i++) {
+        /*
+         * Matching
+         */
+        match = FALSE;
+        if (ssl_cmd_rewrite_map[i].cpCommand != NULL) {
+            if (strcEQ(ssl_cmd_rewrite_map[i].cpCommand, caCmd)) 
+                match = TRUE;
         }
-        if (ssl_cmd_rewrite_map[i].nType == CRM_TXTMSG) {
-            if ((cp = strstr(oline, ssl_cmd_rewrite_map[i].cpMatch)) != NULL) {
+        else if (ssl_cmd_rewrite_map[i].cpSubstring != NULL) {
+            if (strstr(oline, ssl_cmd_rewrite_map[i].cpSubstring) != NULL) 
+                match = TRUE;
+        }
+        else if (ssl_cmd_rewrite_map[i].cpPattern != NULL) {
+            if (ap_fnmatch(ssl_cmd_rewrite_map[i].cpPattern, oline, 0)) 
+                match = TRUE;
+        }
+
+        /*
+         * Action Processing
+         */
+        if (match) {
+            if (ssl_cmd_rewrite_map[i].cpMessage != NULL) {
                 ap_log_error(APLOG_MARK, APLOG_WARNING|APLOG_NOERRNO, s, 
-                             "mod_ssl: Obsolete/unsupported directive: %s",
-                             oline);
-                ap_log_error(APLOG_MARK, APLOG_WARNING|APLOG_NOERRNO, s, 
-                             "mod_ssl: Hint: %s",
-                             (char *)(ssl_cmd_rewrite_map[i].vpSubst));
+                             "mod_ssl:Compat: OBSOLETE '%s' => %s",
+                             oline, ssl_cmd_rewrite_map[i].cpMessage);
                 line = "";
                 break;
             }
-        }
-        else if (ssl_cmd_rewrite_map[i].nType == CRM_PATFCT) {
-            if (ap_fnmatch(ssl_cmd_rewrite_map[i].cpMatch, oline, 0) == 0) {
-                line = ((char *(*)(pool *, const char *))
-                        (ssl_cmd_rewrite_map[i].vpSubst))(p, oline);
+            else if (ssl_cmd_rewrite_map[i].cpSubst != NULL) {
+                if (ssl_cmd_rewrite_map[i].cpCommand != NULL)
+                    line = ap_pstrcat(p, ssl_cmd_rewrite_map[i].cpSubst, 
+                                      cpArgs, NULL);
+                else if (ssl_cmd_rewrite_map[i].cpSubstring != NULL)
+                    line = ssl_util_ptxtsub(p, oline, ssl_cmd_rewrite_map[i].cpSubstring,
+                                            ssl_cmd_rewrite_map[i].cpSubst);
+                else
+                    line = ssl_cmd_rewrite_map[i].cpSubst;
+                break;
+            }
+            else if (ssl_cmd_rewrite_map[i].fpSubst != NULL) {
+                line = ((char *(*)(pool *, const char *, const char *, const char *))
+                        (ssl_cmd_rewrite_map[i].fpSubst))(p, oline, caCmd, cpArgs);
                 break;
             }
         }
     }
+    if (line != NULL && line[0] != NUL)
+        ap_log_error(APLOG_MARK, APLOG_INFO|APLOG_NOERRNO, s, 
+                     "mod_ssl:Compat: MAPPED '%s' => '%s'", oline, line);
     return line;
 }
 
