@@ -149,11 +149,10 @@ void *ssl_config_server_create(pool *p, server_rec *s)
     
     sc = ap_palloc(p, sizeof(SSLSrvConfigRec));
     sc->bEnabled               = UNSET;
-    sc->szCertificateFile      = CFGDEF_CERTIFCATE_FILE;
+    sc->szCertificateFile      = SSL_CERTIFCATE_FILE;
     sc->szKeyFile              = NULL;
     sc->szCACertificatePath    = NULL;
     sc->szCACertificateFile    = NULL;
-    sc->szCACertificateReqFile = NULL;
     sc->szLogFile              = NULL;
     sc->szCipherSuite          = NULL;
     sc->nLogLevel              = SSL_LOG_NONE;
@@ -185,7 +184,6 @@ void *ssl_config_server_merge(pool *p, void *basev, void *addv)
     cfgMergeString(szKeyFile);
     cfgMergeString(szCACertificatePath);
     cfgMergeString(szCACertificateFile);
-    cfgMergeString(szCACertificateReqFile);
     cfgMergeString(szLogFile);
     cfgMergeString(szCipherSuite);
     cfgMerge(nLogLevel, SSL_LOG_NONE);
@@ -306,7 +304,8 @@ const char *ssl_cmd_SSLPassPhraseDialog(
         sc->nPassPhraseDialogType  = SSL_PPTYPE_FILTER;
         sc->szPassPhraseDialogPath = ap_server_root_relative(cmd->pool, arg);
         if (!ssl_util_path_check(SSL_PCM_EXISTS, sc->szPassPhraseDialogPath))
-            return "SSLPassPhraseDialig: file not exists";
+            return ap_pstrcat(cmd->pool, "SSLPassPhraseDialog: file '",
+                              sc->szPassPhraseDialogPath, "' not exists", NULL);
     }
     else 
         return "SSLPassPhraseDialog: Invalid argument";
@@ -338,7 +337,8 @@ const char *ssl_cmd_SSLCertificateFile(
 
     cpPath = ap_server_root_relative(cmd->pool, arg);
     if (!ssl_util_path_check(SSL_PCM_EXISTS|SSL_PCM_ISREG|SSL_PCM_ISNONZERO, cpPath))
-        return "SSLCertificateFile: file not exists or empty";
+        return ap_pstrcat(cmd->pool, "SSLCertificateFile: file '",
+                          cpPath, "' not exists or empty", NULL);
     sc->szCertificateFile = cpPath;
     return NULL;
 }
@@ -351,7 +351,8 @@ const char *ssl_cmd_SSLCertificateKeyFile(
 
     cpPath = ap_server_root_relative(cmd->pool, arg);
     if (!ssl_util_path_check(SSL_PCM_EXISTS|SSL_PCM_ISREG|SSL_PCM_ISNONZERO, cpPath))
-        return "SSLCertificateKeyFile: file not exists or empty";
+        return ap_pstrcat(cmd->pool, "SSLCertificateKeyFile: file '",
+                          cpPath, "' not exists or empty", NULL);
     sc->szKeyFile = cpPath;
     return NULL;
 }
@@ -364,7 +365,8 @@ const char *ssl_cmd_SSLCACertificatePath(
 
     cpPath = ap_server_root_relative(cmd->pool, arg);
     if (!ssl_util_path_check(SSL_PCM_EXISTS|SSL_PCM_ISDIR, cpPath))
-        return "SSLCACertificatePath: directory not exists";
+        return ap_pstrcat(cmd->pool, "SSLCACertificatePath: directory '",
+                          cpPath, "' not exists", NULL);
     sc->szCACertificatePath = cpPath;
     return NULL;
 }
@@ -377,24 +379,12 @@ const char *ssl_cmd_SSLCACertificateFile(
 
     cpPath = ap_server_root_relative(cmd->pool, arg);
     if (!ssl_util_path_check(SSL_PCM_EXISTS|SSL_PCM_ISREG|SSL_PCM_ISNONZERO, cpPath))
-        return "SSLCACertificateFile: file not exists or empty";
+        return ap_pstrcat(cmd->pool, "SSLCACertificateKeyFile: file '",
+                          cpPath, "' not exists or empty", NULL);
     sc->szCACertificateFile = cpPath;
     return NULL;
 }
 
-const char *ssl_cmd_SSLCACertificateReqFile(
-    cmd_parms *cmd, char *struct_ptr, char *arg)
-{
-    SSLSrvConfigRec *sc = mySrvConfig(cmd->server);
-    char *cpPath;
-
-    cpPath = ap_server_root_relative(cmd->pool, arg);
-    if (!ssl_util_path_check(SSL_PCM_EXISTS|SSL_PCM_ISREG|SSL_PCM_ISNONZERO, cpPath))
-        return "SSLCACertificateReqFile: file not exists or empty";
-    sc->szCACertificateReqFile = cpPath;
-    return NULL;
-}
-
 const char *ssl_cmd_SSLVerifyClient(
     cmd_parms *cmd, char *struct_ptr, char *level)
 {
@@ -486,6 +476,8 @@ const char *ssl_cmd_SSLLogLevel(
         sc->nLogLevel = SSL_LOG_WARN;
     else if (strcEQ(level, "info"))
         sc->nLogLevel = SSL_LOG_INFO;
+    else if (strcEQ(level, "trace"))
+        sc->nLogLevel = SSL_LOG_TRACE;
     else if (strcEQ(level, "debug"))
         sc->nLogLevel = SSL_LOG_DEBUG;
     else
