@@ -470,6 +470,7 @@ int ssl_hook_Handler(request_rec *r)
  */
 int ssl_hook_Auth(request_rec *r)
 {
+    SSLSrvConfigRec *sc = mySrvConfig(r->server);
     SSLDirConfigRec *dc = myDirConfig(r);
     char b1[MAX_STRING_LEN], b2[MAX_STRING_LEN];
     char *clientdn;
@@ -477,7 +478,7 @@ int ssl_hook_Auth(request_rec *r)
     /*
      * We decline operation in various situations..
      */
-    if (OPTIONAL_SSL)
+    if (!sc->bEnabled)
         return DECLINED;
     if (!(dc->nOptions & SSL_OPT_FAKEBASICAUTH))
         return DECLINED;
@@ -530,7 +531,7 @@ int ssl_hook_Access(request_rec *r)
     /* 
      * Check to see if SSL protocol is on 
      */
-    if (!sc->bEnabled || OPTIONAL_SSL)
+    if (!sc->bEnabled)
         return DECLINED;
 
     /*
@@ -636,7 +637,7 @@ int ssl_hook_Fixup(request_rec *r)
     /* 
      * Check to see if SSL is on
      */
-    if (!sc->bEnabled || OPTIONAL_SSL)
+    if (!sc->bEnabled)
         return DECLINED;
 
     /*
