@@ -115,6 +115,7 @@ static struct {
     { SSL_LOG_ERROR, "error" },
     { SSL_LOG_WARN,  "warn"  },
     { SSL_LOG_INFO,  "info"  },
+    { SSL_LOG_TRACE, "trace" },
     { SSL_LOG_DEBUG, "debug" },
     { 0, NULL }
 };
@@ -152,6 +153,7 @@ void ssl_log(server_rec *s, int level, const char *msg, ...)
     char lstr[20];
     char vstr[1024];
     char str[1024];
+    char nstr[2];
     int timz;
     struct tm *t;
     va_list ap;
@@ -182,14 +184,34 @@ void ssl_log(server_rec *s, int level, const char *msg, ...)
         return;
 
     /*  determine the time entry string  */
-    t = ap_get_gmtoff(&timz);
-    strftime(tstr, 80, "[%d/%b/%Y %H:%M:%S]", t);
+    if (add & SSL_NO_TIMESTAMP)
+        tstr[0] = NUL;
+    else {
+        t = ap_get_gmtoff(&timz);
+        strftime(tstr, 80, "[%d/%b/%Y %H:%M:%S] ", t);
+    }
+
+    /*  determine whether newline should be writteni  */
+    if (add & SSL_NO_NEWLINE)
+        nstr[0] = NUL;
+    else {
+        nstr[0] = '\n';
+        nstr[1] = NUL;
+    }
 
     /*  determine level name  */
-    ap_cpystrn(lstr, "[???]", sizeof(lstr));
-    for (i = 0; ssl_log_level2string[i].nLevel != 0; i++)
-        if (ssl_log_level2string[i].nLevel == level)
-            ap_snprintf(lstr, sizeof(lstr), "[%s]", ssl_log_level2string[i].szLevel);
+    lstr[0] = NUL;
+    if (!(add & SSL_NO_LEVELID)) {
+        for (i = 0; ssl_log_level2string[i].nLevel != 0; i++) {
+            if (ssl_log_level2string[i].nLevel == level) {
+                ap_snprintf(lstr, sizeof(lstr), "[%s]", ssl_log_level2string[i].szLevel);
+                break;
+            }
+        }
+        for (i = strlen(lstr); i <= 7; i++)
+            lstr[i] = ' ';
+        lstr[i] = NUL;
+    }
     
     /*  create custom message  */
     ap_vsnprintf(vstr, sizeof(vstr), msg, ap);
@@ -204,7 +226,7 @@ void ssl_log(server_rec *s, int level, const char *msg, ...)
     else
         astr = "";
     if (level <= sc->nLogLevel && sc->fileLogFile != NULL) {
-        ap_snprintf(str, sizeof(str), "%s %-7s %s%s\n", tstr, lstr, vstr, astr);
+        ap_snprintf(str, sizeof(str), "%s%s%s%s%s", tstr, lstr, vstr, astr, nstr);
         fprintf(sc->fileLogFile, "%s", str);
     }
     if (level & SSL_LOG_ERROR)
@@ -214,8 +236,8 @@ void ssl_log(server_rec *s, int level, const char *msg, ...)
     /*  write out additional attachment messages  */
     if (add & SSL_ADD_ERRNO) {
         if (level <= sc->nLogLevel && sc->fileLogFile != NULL) {
-            ap_snprintf(str, sizeof(str), "%s %-7s System: %s (errno: %d)\n", 
-                        tstr, lstr, strerror(safe_errno), safe_errno);
+            ap_snprintf(str, sizeof(str), "%s%sSystem: %s (errno: %d)%s", 
+                        tstr, lstr, strerror(safe_errno), safe_errno, nstr);
             fprintf(sc->fileLogFile, "%s", str);
         }
         if (level & SSL_LOG_ERROR)
@@ -228,10 +250,11 @@ void ssl_log(server_rec *s, int level, const char *msg, ...)
             cpE = ERR_error_string(e, NULL);
             cpA = ssl_log_annotation(cpE);
             if (level <= sc->nLogLevel && sc->fileLogFile != NULL) {
-                ap_snprintf(str, sizeof(str), "%s %-7s SSLeay: %s%s%s%s\n", 
+                ap_snprintf(str, sizeof(str), "%s%sSSLeay: %s%s%s%s%s", 
                             tstr, lstr, cpE,
                             cpA != NULL ? " [Hint: " : "", 
-                            cpA != NULL ? cpA : "", cpA != NULL ? "]" : "");
+                            cpA != NULL ? cpA : "", cpA != NULL ? "]" : "",
+                            nstr);
                 fprintf(sc->fileLogFile, "%s", str);
             }
             if (level & SSL_LOG_ERROR)
