@@ -260,16 +260,16 @@ void ssl_pphrase_Handle(server_rec *s, pool *p)
          * Log the type of reading
          */
         if (nPassPhraseDialogCur == 0)
-            ssl_log(pServ, SSL_LOG_DEBUG, 
+            ssl_log(pServ, SSL_LOG_TRACE, 
                     "Init: (%s) unencrypted private key - pass phrase not required", 
                     cpServerName);
         else {
             if (cpPassPhraseCur != NULL)
-                ssl_log(pServ, SSL_LOG_DEBUG, 
+                ssl_log(pServ, SSL_LOG_TRACE, 
                         "Init: (%s) encrypted private key - pass phrase requested", 
                         cpServerName);
             else
-                ssl_log(pServ, SSL_LOG_DEBUG, 
+                ssl_log(pServ, SSL_LOG_TRACE, 
                         "Init: (%s) encrypted private key - pass phrase reused", 
                         cpServerName);
         }
@@ -384,12 +384,12 @@ int ssl_pphrase_Handle_CB(char *buf, int bufsize, int w)
             fprintf(stderr, "%s mod_ssl/%s (Pass Phrase Dialog)\n",
                     SERVER_BASEVERSION, MOD_SSL_VERSION);
             fprintf(stderr, "Some of your private key files are encrypted for security reasons.\n");
-            fprintf(stderr, "In order to read them you have to provide with us the pass phrases.\n");
+            fprintf(stderr, "In order to read them you have to provide us with the pass phrases.\n");
         }
         if (*pbPassPhraseDialogOnce) {
             *pbPassPhraseDialogOnce = FALSE;
             fprintf(stderr, "\n");
-            fprintf(stderr, "Server: %s\n", cpServerName);
+            fprintf(stderr, "Server %s\n", cpServerName);
         }
 
         /* 
