@@ -261,7 +261,7 @@ void ssl_scache_dbm_store(SessionCacheInfo *SCI)
     dbmval.dsize = sizeof(SCI->tExpiresAt)+SCI->nData;
     dbmval.dptr  = (UCHAR *)malloc(dbmval.dsize);
     memcpy(dbmval.dptr, &SCI->tExpiresAt, sizeof(SCI->tExpiresAt));
-    memcpy(dbmval.dptr+sizeof(SCI->tExpiresAt), SCI->ucaData, SCI->nData);
+    memcpy((char *)dbmval.dptr+sizeof(SCI->tExpiresAt), SCI->ucaData, SCI->nData);
 
     /* and store it to the DBM file */
     ssl_mutex_on();
@@ -320,7 +320,7 @@ void ssl_scache_dbm_retrieve(SessionCacheInfo *SCI)
     /* copy over the information to the SCI */
     SCI->nData   = dbmval.dsize-sizeof(SCI->tExpiresAt);
     SCI->ucaData = (UCHAR *)malloc(dbmval.dsize-sizeof(SCI->tExpiresAt));
-    memcpy(SCI->ucaData, dbmval.dptr+sizeof(SCI->tExpiresAt), 
+    memcpy(SCI->ucaData, (char *)dbmval.dptr+sizeof(SCI->tExpiresAt), 
            dbmval.dsize-sizeof(SCI->tExpiresAt));
     memcpy(&SCI->tExpiresAt, dbmval.dptr, sizeof(SCI->tExpiresAt));
 
