@@ -306,6 +306,7 @@ static char *ssl_var_lookup_ssl_cert(pool *p, X509 *xs, char *var)
     BOOL resdup;
     X509_NAME *xsname;
     int nid;
+    char *cp;
     
     result = NULL;
     resdup = TRUE;
@@ -325,7 +326,10 @@ static char *ssl_var_lookup_ssl_cert(pool *p, X509 *xs, char *var)
     }
     else if (strcEQ(var, "S_DN")) {
         xsname = X509_get_subject_name(xs);
-        result = X509_NAME_oneline(xsname, NULL, 0);
+        cp = X509_NAME_oneline(xsname, NULL, 0);
+        result = ap_pstrdup(p, cp);
+        free(cp);
+        resdup = FALSE;
     }
     else if (strlen(var) > 5 && strcEQn(var, "S_DN_", 5)) {
         xsname = X509_get_subject_name(xs);
@@ -334,7 +338,10 @@ static char *ssl_var_lookup_ssl_cert(pool *p, X509 *xs, char *var)
     }
     else if (strcEQ(var, "I_DN")) {
         xsname = X509_get_issuer_name(xs);
-        result = X509_NAME_oneline(xsname, NULL, 0);
+        cp = X509_NAME_oneline(xsname, NULL, 0);
+        result = ap_pstrdup(p, cp);
+        free(cp);
+        resdup = FALSE;
     }
     else if (strlen(var) > 5 && strcEQn(var, "I_DN_", 5)) {
         xsname = X509_get_issuer_name(xs);
@@ -389,7 +396,7 @@ static char *ssl_var_lookup_ssl_cert_dn(pool *p, X509_NAME *xsname, char *var)
                 n = OBJ_obj2nid(xsne->object);
                 if (n == ssl_var_lookup_ssl_cert_dn_rec[i].nid) {
                     result = ap_palloc(p, xsne->value->length+1);
-                    ap_cpystrn(result, xsne->value->data, xsne->value->length+1);
+                    ap_cpystrn(result, (char *)xsne->value->data, xsne->value->length+1);
                     result[xsne->value->length] = NUL;
                     break;
                 }
